# -*- coding: utf-8 -*-
import magic

from django.contrib import messages
from django.http import HttpResponse, HttpResponseRedirect
from django.urls import reverse
from django.utils.translation import gettext_lazy as _
from django.views import View
from django.views.generic import FormView

from .forms import ImportJobForm, ExportJobForm, SendExportJobForm
from ..utils import generate_filename, delete_private_file, write_private_file


class ValidateAttributeMixin:
    def get_title(self):
        """Return title used from template and breadcrumbs"""
        if self.title is None:
            raise AttributeError(_("'title' attribute not set for {}").format(self.__class__.__name__))
        return self.title

    def get_form_class(self):
        """Return the form class to use."""
        if not self.form_class:
            return ImportJobForm
        return self.form_class

    def get_job_class(self):
        """Return the job class to use."""
        if self.job_class is None:
            raise AttributeError(_("'job_class' attribute not set for {}").format(self.__class__.__name__))
        return self.job_class

    def get_app_label(self):
        """Return app label to use."""
        if self.app_label is None:
            raise AttributeError(_("'app_label' attribute not set for {}").format(self.__class__.__name__))
        return self.app_label

    def get_model_name(self):
        """Return model name to use."""
        if self.model_name is None:
            raise AttributeError(_("'model_name' attribute not set for {}").format(self.__class__.__name__))
        return self.model_name

    def get_app_title(self):
        """Return app title to use in breadcrumbs."""
        if self.app_title is None:
            raise AttributeError(_("'app_title' attribute not set for {}").format(self.__class__.__name__))
        return self.app_title

    def get_model_title(self):
        """Return model title to use in breadcrumbs"""
        if self.model_title is None:
            raise AttributeError(_("'model_title' attribute not set for {}").format(self.__class__.__name__))
        return self.model_title

    def get_url(self):
        """Return url to use for template path"""
        if self.url is None:
            raise AttributeError(_("'url' attribute not set for {}").format(self.__class__.__name__))
        return self.url

    def get_model(self):
        """Return Model class to use."""
        if self.model is None:
            raise AttributeError(_("'model' attribute not set for {}").format(self.__class__.__name__))
        return self.model

    def get_prefix_path(self):
        """Return prefix path to set media private root"""
        if self.prefix_path is None:
            raise AttributeError(_("'prefix_path' attribute not set for {}").format(self.__class__.__name__))
        return self.prefix_path

    def get_delete(self):
        """Return True or False if file have to be deleted after download"""
        if self.delete is None:
            raise AttributeError(_("'delete' attribute not set for {}").format(self.__class__.__name__))
        return self.delete


class ImportJobView(ValidateAttributeMixin, FormView):
    """
    A view for displaying a form to upload file and create job to import file.
    """
    title = None
    form_class = None
    job_class = None
    app_label = None
    model_name = None
    app_title = None
    model_title = None
    url = None
    model = None
    template_name = None

    def get_template_names(self):
        """
        Return a list of template names to be used for the request.
        """
        app_label = self.get_app_label()
        model_name = self.get_model_name()
        url = self.get_url()
        template_name = f"admin/{app_label}/{model_name}/{url}.html"
        return [
            'admin/import_export_change_form.html',
            template_name
        ]

    def get_breadcrumbs(self):
        """
        Return a list of breadcrumbs to be used.
        """
        app_label = self.get_app_label()
        model_name = self.get_model_name()
        title = self.get_title()
        app_title = self.get_app_title()
        model_title = self.get_model_title()
        return (
            (_('Home'), reverse('admin:index')),
            (app_title, reverse('admin:app_list', kwargs={'app_label': '{}'.format(app_label)})),
            (model_title, reverse('admin:{}_{}_changelist'.format(app_label, model_name))),
            (title,),
        )

    def get_context_data(self, **kwargs):
        """
        Override default context with add breadcrumbs and success url.
        """
        context = super().get_context_data(**kwargs)
        model = self.get_model()
        opts = model._meta
        title = self.get_title()
        breadcrumbs = self.get_breadcrumbs()
        success_url = self.get_success_url()
        context.update({
            'opts': opts,
            'title': title,
            'breadcrumbs': breadcrumbs,
            'success_url': success_url
        })
        return context

    def get_success_url(self):
        """
        Return "success url" to use.
        """
        app_label = self.get_app_label()
        model_name = self.get_model_name()
        return reverse('admin:{}_{}_changelist'.format(app_label, model_name))

    def get_parameters(self, form, filename_or_error):
        """
        Return a dict of parameters to pass to job "create".
        """
        return dict(
            user_id=self.request.user.id,
            filename=filename_or_error,
        )

    def form_valid(self, form):
        """
        If the form is valid, method:
        - writes file on private path
        - creates job to import file
        """
        user = self.request.user
        created, filename_or_error = write_private_file(user, form.cleaned_data['filename'])
        if created:
            messages.success(self.request, _('file successfully uploaded'))
            parameters = self.get_parameters(form, filename_or_error)
            job_class = self.get_job_class()
            created, job_or_error = job_class.create(creator=user, parameters=parameters)
            if created:
                # url = reverse('admin:cron_job_change', args=[job_or_error.pk])
                # link = f"<a href='{url}'>{job_or_error.pk}</a>"
                messages.success(self.request, _('job {} successfully created').format(job_or_error.pk))
            else:
                messages.error(self.request, job_or_error)
        else:
            messages.error(self.request, filename_or_error)
        return super().form_valid(form)


class ExportJobView(ValidateAttributeMixin, FormView):
    """
    A view for displaying a form to create job and export file.
    """
    title = None
    form_class = ExportJobForm
    job_class = None
    app_label = None
    model_name = None
    app_title = None
    model_title = None
    url = None
    model = None
    template_name = None

    def get_template_names(self):
        """
        Return a list of template names to be used for the request.
        """
        app_label = self.get_app_label()
        model_name = self.get_model_name()
        url = self.get_url()
        template_name = f"admin/{app_label}/{model_name}/{url}.html"
        return [
            'admin/import_export_change_form.html',
            template_name
        ]

    def get_breadcrumbs(self):
        """
        Return a list of breadcrumbs to be used.
        """
        app_label = self.get_app_label()
        model_name = self.get_model_name()
        title = self.get_title()
        app_title = self.get_app_title()
        model_title = self.get_model_title()
        return (
            (_('Home'), reverse('admin:index')),
            (app_title, reverse('admin:app_list', kwargs={'app_label': '{}'.format(app_label)})),
            (model_title, reverse('admin:{}_{}_changelist'.format(app_label, model_name))),
            (title,),
        )

    def get_context_data(self, **kwargs):
        """
        Override default context with add breadcrumbs and success url.
        """
        context = super().get_context_data(**kwargs)
        model = self.get_model()
        opts = model._meta
        title = self.get_title()
        breadcrumbs = self.get_breadcrumbs()
        success_url = self.get_success_url()
        context.update({
            'opts': opts,
            'title': title,
            'breadcrumbs': breadcrumbs,
            'success_url': success_url
        })
        return context

    def get_success_url(self):
        """
        Return "success url" to use.
        """
        app_label = self.get_app_label()
        model_name = self.get_model_name()
        return reverse('admin:{}_{}_changelist'.format(app_label, model_name))

    def get_parameters(self, form):
        """
        Return a dict of parameters to pass to job "create".
        """
        parameters = dict(
            user_id=self.request.user.id,
        )
        if form.cleaned_data['file_name']:
            parameters['file_name'] = form.cleaned_data['file_name']
        return parameters

    def form_valid(self, form):
        """
        If the form is valid, method
        - creates job to export file
        """
        user = self.request.user
        parameters = self.get_parameters(form)
        created, job_or_error = self.job_class.create(creator=user, parameters=parameters)
        if created:
            messages.success(self.request, _('job {} successfully created').format(job_or_error.pk))
        else:
            messages.error(self.request, job_or_error)
        return super().form_valid(form)


class SendExportJobView(ExportJobView):
    """
    A view for displaying a form to create job, export file and send email.
    """
    form_class = SendExportJobForm

    def get_parameters(self, form):
        """
        Return a dict of parameters to pass to job "create" method.
        """
        parameters = dict(
            user_id=self.request.user.id,
            email=form.cleaned_data['email'],
            subject=form.cleaned_data['subject'],
        )
        if form.cleaned_data['file_name']:
            parameters['file_name'] = form.cleaned_data['file_name']
        return parameters


class DownloadView(ValidateAttributeMixin, View):
    """
    A view for download file w/o authentication.
    """
    prefix_path = None
    delete = True
    app_label = None
    model_name = None
    file_name = None
    file_content = None

    def get_file_name(self):
        """Return only file name"""
        file_name = self.kwargs.get('file_name')
        if file_name is None:
            raise AttributeError(_("'file_name' attribute not set for {}").format(self.__class__.__name__))
        return file_name

    def get_filename(self):
        """Return file name with path"""
        user = self.get_user()
        file_name = self.get_file_name()
        prefix_path = self.get_prefix_path()
        return generate_filename(user, file_name, prefix_path)

    def get_success_url(self):
        """
        Return "success url" to use.
        """
        app_label = self.get_app_label()
        model_name = self.get_model_name()
        return reverse('admin:{}_{}_changelist'.format(app_label, model_name))

    def get_file_content(self):
        """Return file content"""
        filename = self.get_filename()
        with open(filename, 'rb') as f:
            return f.read()

    def render_to_response(self):
        """
        Return a response with file content
        """
        user = self.get_user()
        file_content = self.get_file_content()
        file_name = self.get_file_name()
        filename = self.get_filename()
        prefix_path = self.get_prefix_path()
        success_url = self.get_success_url()
        try:
            content_type = magic.from_file(filename, mime=True)
            response = HttpResponse(file_content, content_type=content_type)
            response["Content-Disposition"] = f'attachment;filename="{file_name}"'
            if self.delete:
                delete_private_file(user, filename, prefix_path)
            return response
        except Exception as e:
            # TODO gestire meglio l'eccezione
            messages.error(self.request, _('File not found'))
            return HttpResponseRedirect(success_url)

    def get(self, request, *args, **kwargs):
        return self.render_to_response()

    def get_user(self):
        return self.request.user
